@@ -17,13 +17,14 @@
 #include "properties/Integer.hh"
 #include "properties/SortOrder.hh"
 
-//#define DEBUG 1
+// #define DEBUG 1
 
+static bool debug_stop = false;
 
 #ifdef DEBUG
-  #define DEBUGLN(ln) ln
+#define DEBUGLN(ln) if(!debug_stop) { ln; }
 #else
-  #define DEBUGLN(ln)
+#define DEBUGLN(ln)
 #endif
 
 namespace cadabra {
@@ -304,6 +305,18 @@ namespace cadabra {
 	Ex_comparator::match_t Ex_comparator::equal_subtree(Ex::iterator i1, Ex::iterator i2,
 	      useprops_t use_props, bool ignore_parent_rel)
 		{
+		// FIXME: Ex_comparator should really know about the tree,
+		// so that we can do safety checks like the ones below.
+		// 
+		// if(!i1.is_valid() && i2.is_valid())
+		// 	return match_t::no_match_less;
+		// 
+		// if(i1.is_valid() && !i2.is_valid())
+		// 	return match_t::no_match_greater;
+		// 
+		// if(!i1.is_valid() && !i2.is_valid())
+		// 	return match_t::subtree_match;
+		
 		DEBUGLN( std::cerr << tab() << "equal_subtree with use_props = " << use_props << std::endl; );
 		++offset;
 
@@ -440,34 +453,37 @@ namespace cadabra {
 		return r;
 #endif
 
-		std::cerr << tab() << "result = ";
-		switch(r) {
-			case match_t::node_match:
-				std::cerr << "node_match";
-				break;
-			case match_t::subtree_match:
-				std::cerr << "subtree_match";
-				break;
-			case match_t::match_index_less:
-				std::cerr << "match_index_less";
-				break;
-			case match_t::match_index_greater:
-				std::cerr << "match_index_greater";
-				break;
-			case match_t::no_match_indexpos_less:
-				std::cerr << "no_match_indexpos_less";
-				break;
-			case match_t::no_match_indexpos_greater:
-				std::cerr << "no_match_indexpos_greater";
-				break;
-			case match_t::no_match_less:
-				std::cerr << "no_match_less";
-				break;
-			case match_t::no_match_greater:
-				std::cerr << "no_match_greater";
-				break;
+		DEBUGLN( std::cerr << tab() << "result = "; );
+		if(!debug_stop) {
+			switch(r) {
+				case match_t::node_match:
+					std::cerr << "node_match";
+					break;
+				case match_t::subtree_match:
+					std::cerr << "subtree_match";
+					break;
+				case match_t::match_index_less:
+					std::cerr << "match_index_less";
+					break;
+				case match_t::match_index_greater:
+					std::cerr << "match_index_greater";
+					break;
+				case match_t::no_match_indexpos_less:
+					std::cerr << "no_match_indexpos_less";
+					break;
+				case match_t::no_match_indexpos_greater:
+					std::cerr << "no_match_indexpos_greater";
+					break;
+				case match_t::no_match_less:
+					std::cerr << "no_match_less";
+					break;
+				case match_t::no_match_greater:
+					std::cerr << "no_match_greater";
+					break;
+				}
+			std::cerr << std::endl;
 			}
-		std::cerr << std::endl;
+		
 		--offset;
 		return r;
 		}
@@ -495,6 +511,7 @@ namespace cadabra {
 		bool is_sibling_pattern=false;
 		bool is_coordinate=false;
 		bool is_number=false;
+		bool two_is_value=false;
 
 		if(one->fl.bracket==str_node::b_none && one->is_index() )
 			is_index=true;
@@ -505,7 +522,8 @@ namespace cadabra {
 		else if(one->is_siblings_wildcard())
 			is_sibling_pattern=true;
 		else if(is_index && one->is_integer()==false) {
-			// Things in _{..} or ^{..} are either indices (implicit patterns) or coordinates.
+			// Things in _{..} or ^{..} are either indices (implicit patterns), or coordinates (as
+			// numbers are treated in the last branch below).
 			const Coordinate *cdn1=0;
 			if(use_props==useprops_t::always) {
 				DEBUGLN( std::cerr << tab() << "is " << *one->name << " a coordinate?" << std::endl; );
@@ -536,7 +554,7 @@ namespace cadabra {
 
 			if(two->is_rational() && implicit_pattern) {
 				// Determine whether 'one' can take the value 'two'.
-				//std::cerr << "**** can one take value two " << use_props  << std::endl;
+				DEBUGLN( std::cerr << tab() << "can one take value two (use props = " << use_props  << ")" << std::endl; );
 				const Integer *ip = 0;
 				if(use_props==useprops_t::always) {
 					DEBUGLN( std::cerr << tab() << "is " << *one->name << " an integer?" << std::endl; );
@@ -545,6 +563,8 @@ namespace cadabra {
 					}
 
 				if(ip==0) return report(match_t::no_match_less);
+				
+				DEBUGLN( std::cerr << tab() << "yes, potentially, check the range" << std::endl; );
 
 				bool lower_bdy=true, upper_bdy=true;
 				multiplier_t from, to;
@@ -561,7 +581,7 @@ namespace cadabra {
 				if((lower_bdy && *two->multiplier < from) || (upper_bdy && *two->multiplier > to))
 					return report(match_t::no_match_less);
 
-				// std::cerr << tab() << Ex(one) << tab() << "can take value " << *two->multiplier << std::endl;
+				DEBUGLN( std::cerr << tab() << Ex(one) << tab() << "can take value " << *two->multiplier << std::endl );
 				}
 
 			// We want to search the replacement map for replacement rules which we have
@@ -630,13 +650,17 @@ namespace cadabra {
 					const Indices *t1=0;
 					const Indices *t2=0;
 					if(use_props==useprops_t::always) {
-						DEBUGLN( std::cerr << tab() << "is " << one << " an index?" << std::endl; );
+						DEBUGLN( std::cerr << tab() << "is " << *one->name << " an index?" << std::endl; );
+						debug_stop=true;
 						t1=properties.get<Indices>(one, false);
-						DEBUGLN( std::cerr << tab() << "found for one: " << t1 << std::endl; );
+						debug_stop=false;						
+						DEBUGLN( std::cerr << tab() << "found index property for one: " << t1 << std::endl; );
 						if(two->is_rational()==false) {
-							DEBUGLN( std::cerr << tab() << "is " << two << " an index?" << std::endl; );
+							DEBUGLN( std::cerr << tab() << "is " << *two->name << " an index?" << std::endl; );
+							debug_stop=true;
 							t2=properties.get<Indices>(two, false);
-							DEBUGLN( std::cerr << tab() << t2 << std::endl; );
+							debug_stop=false;
+							DEBUGLN( std::cerr << tab() << "found index property for two: " << t2 << std::endl; );
 							// It is still possible that t2 is a Coordinate and
 							// t1 an Index which can take the value of the
 							// coordinate. This happens when 'm' is an index
@@ -653,8 +677,10 @@ namespace cadabra {
 									if(subtree_compare(&properties, a.begin(), two, 0)==0) return true;
 									else return false;
 								});
-								if(ivals!=t1->values.end())
+								if(ivals!=t1->values.end()) {
 									t2=t1;
+									two_is_value=true;
+									}
 								}
 							}
 							else {
@@ -757,10 +783,18 @@ namespace cadabra {
 			// they matched because they are from the same set but do not have the same
 			// name, we still need to let the caller know about this.
 			if(is_index) {
-				int xc = subtree_compare(0, one, two, ignore_parent_rel?(-1):(-2));
-				if(xc==0) return report(match_t::subtree_match);
-				if(xc>0)  return report(match_t::match_index_less);
-				return report(match_t::match_index_greater);
+				if(two->is_rational() || two_is_value) {
+					// We matched an index in 'one' to a value in 'two', report as matching.
+					DEBUGLN( std::cerr << "Matched index to numerical value." << std::endl; );
+					return report(match_t::subtree_match);
+					}
+				else {
+					DEBUGLN( std::cerr << "Index type matches, report whether names match too." << std::endl; );
+					int xc = subtree_compare(0, one, two, ignore_parent_rel?(-1):(-2));
+					if(xc==0) return report(match_t::subtree_match);
+					if(xc>0)  return report(match_t::match_index_less);
+					return report(match_t::match_index_greater);
+					}
 				}
 			else return report(match_t::node_match);
 			}
