@@ -151,6 +151,11 @@ namespace cadabra {
 		return to_long(*(begin()->multiplier));
 		}
 
+	bool Ex::is_empty() const
+		{
+		return begin()==end();
+		}
+
 	std::ostream& Ex::print_python(std::ostream& str, Ex::iterator it)
 		{
 		std::string name(*(*it).name);
@@ -1009,6 +1014,12 @@ found:
 		num=rat_set.insert(fac).first;
 		}
 
+	void set(rset_t::iterator& num, multiplier_t fac)
+		{
+		fac.canonicalize();
+		num=rat_set.insert(fac).first;
+		}
+
 	void add(rset_t::iterator& num, multiplier_t fac)
 		{
 		fac+=*num;
