@@ -23,9 +23,11 @@ canonicalise::canonicalise(const Kernel& k, Ex& tr)
 bool canonicalise::can_apply(iterator it)
 	{
 	if(*(it->name)!="\\prod")
-		if(is_single_term(it)==false)
+		if(*(it->name)=="\\pow" || is_single_term(it)==false)
 			return false;
 
+	//if(! (tr.is_head(it)==false && *tr.parent(it)->name=="\\pow" && tr.index(it)==0) )
+
 	// Canonicalise requires strict monomial structure: no products which contain
 	// sums as factors. Products as factors are ok, they do not lead to multiple
 	// identically named free indices.
@@ -41,6 +43,9 @@ bool canonicalise::can_apply(iterator it)
 		return false;
 		}
 
+#ifdef DEBUG
+	std::cerr << "canonicalise::can_apply: at " << it << std::endl;
+#endif
 	return true;
 	}
 
@@ -390,6 +395,9 @@ Algorithm::result_t canonicalise::apply(iterator& it)
 			// std::cerr << Ex(facit) << " has tableaubase " << tba << std::endl;
 			if(tba) {
 				unsigned int num_ind=number_of_indices(facit);
+#ifdef XPERM_DEBUG
+				std::cerr << "factor " << *facit->name << " has " << num_ind << " indices" << std::endl;
+#endif
 
 				// Add indices to the base. We used to add everything except the last one, but that
 				// seems to be the wrong thing to do after the XPERM -> XPERM_EXT upgrade (see Jose's email).
