@@ -97,6 +97,7 @@ namespace cadabra {
 
    Ex_ptr Ex_join(const Ex_ptr ex1, const Ex_ptr ex2)
 		{
+		// DEPRECATED: no longer in use, can be removed
 		if (ex1->size() == 0) return ex2;
 		if (ex2->size() == 0) return ex1;
 
@@ -130,6 +131,26 @@ namespace cadabra {
 			return ret;
 			}
 		}
+
+   Ex_ptr Ex_join(const std::vector<Ex_ptr>& exs)
+		{
+		auto ret = std::make_shared<Ex>("\\comma");
+
+		for(const Ex_ptr& ex: exs) {
+			// skip ex if empty
+			if (ex->size() > 0) {
+				auto loc = ret->append_child(ret->begin(), ex->begin());
+				if(*ex->begin()->name=="\\comma") 
+					ret->flatten_and_erase(loc);
+				}
+			}
+		// Note: ret could still \comma{} or \comma{x}
+		auto it = ret->begin();
+		if (ret->number_of_children(it) < 2) {
+			ret->flatten_and_erase(it);
+			}
+		return ret;
+		}
 	
 	Ex_ptr Ex_mul(const Ex_ptr ex1, const Ex_ptr ex2)
 		{
@@ -370,7 +391,9 @@ namespace cadabra {
 		Ex result;
 
 		pybind11::size_t start, stop, step, length;
-		slice.compute(ex->size(), &start, &stop, &step, &length);
+		// std::cerr << "SLICE object has " << ex->number_of_children(ex->begin()) << " elements" << std::endl;
+		slice.compute(ex->number_of_children(ex->begin()), &start, &stop, &step, &length);
+		// std::cerr << "SLICE: " << start << ", " << stop << ", " << step << ", " << length << std::endl;
 		if (length == 0)
 			return result;
 
@@ -378,14 +401,11 @@ namespace cadabra {
 		auto it = result.set_head(*ex->begin());
 
 		// Iterate over fully-closed range.
-		for (; start != stop; start += step) {
-			Ex::iterator toadd(ex->begin());
+		for (; start < stop; start += step) {
+			Ex::sibling_iterator toadd=ex->begin(ex->begin());
 			std::advance(toadd, start);
-			result.append_child(it, toadd);
+			result.append_child(it, Ex::iterator(toadd));
 			}
-		Ex::iterator toadd(ex->begin());
-		std::advance(toadd, start);
-		result.append_child(it, toadd);
 		return result;
 		}
 
@@ -653,6 +673,7 @@ namespace cadabra {
 		pybind11::class_<ExNode>(m, "ExNode", "Iterator over elements of an Ex mathematical expression.")
 			.def("__iter__", &ExNode::iter)
 			.def("__next__", &ExNode::next, pybind11::return_value_policy::reference_internal)
+			.def("input_form",  &ExNode::input_form)
 			.def("__getitem__", &ExNode::getitem_string)
 			.def("__getitem__", &ExNode::getitem_iterator)
 			.def("__setitem__", &ExNode::setitem_string)
@@ -688,7 +709,14 @@ namespace cadabra {
 		.def("from_sympy", &sympy::SympyBridge::import_ex)
 		;
 
-		m.def("join", &Ex_join);
+		m.def("join", [](const Ex_ptr ex1, const Ex_ptr ex2, py::args args) {
+			std::vector<Ex_ptr> ex = {ex1, ex2};
+        	for (const auto& arg : args) {
+				ex.push_back(arg.cast<Ex_ptr>());
+				}
+        	return Ex_join(ex);
+    		});
+
 		m.def("tree", &print_tree);
 
 		m.def("map_sympy", &map_sympy_wrapper,
