@@ -10,6 +10,7 @@
 #include "py_stopwatch.hh"
 #include "py_tableau.hh"
 #include "py_ntensor.hh"
+#include "py_media.hh"
 
 #include "Kernel.hh"
 
@@ -35,6 +36,7 @@ namespace cadabra {
 		init_kernel(m);
 		init_progress_monitor(m);
 		init_ntensor(m);
+		init_media(m);
 		init_stopwatch(m);
 		init_ex(m);
 		init_tableau(m);
